QuestAnnounce = LibStub("AceAddon-3.0"):NewAddon("QuestAnnounce", "AceConsole-3.0","AceEvent-3.0")
local QuestAnnounce = _G.QuestAnnounce
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

local AceLocale = LibStub("AceLocale-3.0")
local L = AceLocale:GetLocale("QuestAnnounce")

local version = GetBuildInfo()

--[[ The defaults user without a profile will get. --]]
local defaults = {
	profile={
		Debug = false,
		Enabled = true,
		Every = 1
	}
}

--[[ The command line options to pass into AceConsole. --]] 
local options = {
    name = "QuestAnnounce",
    handler = QuestAnnounce,
    type = 'group',
    args = {
		[L["toggle"]] = {
			order = 1,
			name = L["Toggle"],
			desc = L["Enables/Disables Quest Announce"],
			type = 'execute',
			func = function()
				if(QuestAnnounce.db.profile.Enabled) then
					QuestAnnounce.db.profile.Enabled = false
					QuestAnnounce:Print(L["Toggle"].." - "..L["Disabled"])
				else
					QuestAnnounce.db.profile.Enabled = true
					QuestAnnounce:Print(L["Toggle"].." - "..L["Enabled"])
				end
			end
        },
		[L["every"]] = {
			order = 2,
			name = L["Announce Every"],
			desc = L["Announce progression every x number of steps"],
			type = 'range',
			min = 1,
			max = 10,
			step = 1,
			get = function(info) return QuestAnnounce.db.profile.Every or 1 end,
			set = function(info, value)
				QuestAnnounce.db.profile.Every = value
				QuestAnnounce:Print(L["Announce Every"].." - "..tostring(value))
			end,
			validate = function(info, value)  
				if(value > 10) then return L["Announce Every has to be between 1 and 10"] end
			end
        },
		[L["debug"]] = {
			order = 3,
			name = L["Debug"],
			desc = L["Enables/Disables Quest Announce Debug Mode"],
			cmdHidden = true,
			type = 'execute',
			func = function()
				if(QuestAnnounce.db.profile.Debug) then
					QuestAnnounce.db.profile.Debug = false 
					QuestAnnounce:Print(L["Debug"].." - "..L["Disabled"])
				else
					QuestAnnounce.db.profile.Debug = true
					QuestAnnounce:Print(L["Debug"].." - "..L["Enabled"])
				end
			end
        },
		[L["status"]] = {
			order = 4,
			name = L["Status"],
			desc = L["Display the current status of Quest Announce"],
			type = 'execute',
			func = function()
				QuestAnnounce:Print(L["Status"].."\n    "..
					L["Toggle"].." - "..(QuestAnnounce.db.profile.Enabled and L["Enabled"] or L["Disabled"]).."\n    "..
					L["Announce Every"].." - "..tostring(QuestAnnounce.db.profile.Every))
			end
        },
		[L["reset"]] = {
			order = 4,
			name = L["Reset"],
			desc = L["Resets your Quest Announce profile to the default settings"],
			type = 'execute',
			func = function() 
				QuestAnnounce.db:ResetDB("default")
				QuestAnnounce:Print(L["Settings have been reset to the defaults"])
			end
        }
    }
}

function QuestAnnounce:OnInitialize()
	local AceDB = LibStub:GetLibrary("AceDB-3.0")
	QuestAnnounce.db = AceDB:New("QuestAnnounceDB", defaults)
		
	QuestAnnounce:RegisterEvent("UI_INFO_MESSAGE", function(event, ...) QuestAnnounce:OnEvent(event, ...) end)
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("QuestAnnounce", options, {"qa", "questannounce"})
	
	QuestAnnounce:Print(" v."..version)
end

function QuestAnnounce:OnEvent(event, ...)
	local msg = ...
	if (msg ~= nil) then
		if (QuestAnnounce.db.profile.Enabled) then
			local questText = gsub(msg,"(.*):%s*([-%d]+)%s*/%s*([-%d]+)%s*$", "%1", 1);
			
			if (questText ~= msg) then
				local ii, jj, strItemName, iNumItems, iNumNeeded = string.find(msg, "(.*):%s*([-%d]+)%s*/%s*([-%d]+)%s*$");
				local stillNeeded = iNumNeeded - iNumItems;
                
				local every = math.fmod(iNumItems, QuestAnnounce.db.profile.Every)
				QuestAnnounce:SendDebugMsg("Item Name: "..strItemName.." :: Num Items: "..iNumItems.." :: Num Needed: "..iNumNeeded.." :: Every fMod: "..every)
				
				if(every == 0 and stillNeeded > 0 ) then
					QuestAnnounce:SendPartyMsg(L["Progress: "]..msg);
				elseif(stillNeeded == 0) then
					QuestAnnounce:SendPartyMsg(L["Completed: "]..msg);
				end
			end
		end
	end
end

function QuestAnnounce:SendDebugMsg(msg)
	if(msg ~= nil and QuestAnnounce.db.profile.Debug) then
		QuestAnnounce:Print("DEBUG :: "..msg);
	end
end

function QuestAnnounce:SendPartyMsg(msg)	
	if (msg ~= nil and QuestAnnounce.db.profile.Enabled and GetNumPartyMembers() > 0) then
		SendChatMessage(msg, "PARTY");
	end
	QuestAnnounce:SendDebugMsg("QuestAnnounce:SendPartyMsg - "..msg);
end
